/*! 
* BeX5 v3 (http://www.justep.com) 
* Copyright 2015 Justep, Inc.
*/ 
define(function(require) {
	var justep = require("$UI/system/lib/justep");
	var WindowContainer = require("../windowContainer/windowContainer");
	var WindowDialog = require("../windowDialog/windowDialog");

	var url = require.normalizeName("./orgDialogm");
	var ComponentConfig = require("./orgDialogm.config");
	
	var multiOrgDialogURL = require.toUrl('./dialog/orgDialogm.w');
	var orgDialogURL = require.toUrl('./dialog/singleOrgDialogm.w');
	var defaultTitle = justep.Message.getMessage(justep.Message.JUSTEP231089);

	var OrgDialogPC = WindowDialog.extend({
		getConfig : function() {
			return ComponentConfig;
		},
		constructor : function(options) {
			this.limit = 20;
			this.multiSelection = false;
			this.cascade = false;
			this.includeOrgKind = "";
			this.includeDisabled = false;
			this.showCommonGroup = true;
			this.manageCodes = "";
			this.rootFilter = "";
			this.filter = "";
			this.selectFilter = "";
			this.useVirtualRoot = false;
			this.virtualRootLabel = '';

			this.cacheKind = '';
			this.callParent(options);
			this.title = defaultTitle;
		},
		doInit : function(value, bindingContext) {
			this.src = this.multiSelection?multiOrgDialogURL:orgDialogURL;
			this.on(WindowContainer.LOAD_EVENT, this._doLoadOrgData, this);
			this.callParent(value, bindingContext);
			this._getTitleTextNode().text(this.title);
		},
		buildTemplate : function(cfg) {
			var e = this.callParent(cfg);
			e.attr('component', url);
			return e;
		},
		propertyChangedHandler : function(key, oldVal, value) {
			switch (key) {
			case "multiSelection":
				if (oldVal != value && this._inited){
					this._loaded = false;
					if(this._compose){
						this.src = value?multiOrgDialogURL:orgDialogURL;
						this._compose.setSrc(this.src);
					}
				}
				break;
			default:
				this.callParent(key, oldVal, value);
			}
		},
		_doLoadOrgData : function() {
			var innerModel = this._compose.getInnerModel();
			innerModel.open();
		},
		_setDialogParam : function(option) {
			var innerModel = this._compose.getInnerModel(), 
			param = {
				limit : this.limit,
				rootFilter : this.rootFilter,
				filter: this.filter,
				manageCodes : this.manageCodes,
				selectFilter : this.selectFilter,
				includeOrgKind : this.includeOrgKind,
				includeDisabled: this.includeDisabled,
				showCommonGroup: this.showCommonGroup,
				useVirtualRoot: this.useVirtualRoot,
				virtualRootLabel: this.virtualRootLabel,
				cascade: this.cascade,
				cacheKind: this.cacheKind,
				title : this.title
			};
			innerModel.setParam(param);
			if(option && option.reload) innerModel.reload();
		}
	});

	OrgDialogPC.CacheKind = {ORG_CACHE:'ORG_CACHE',ORG_ALL_CACHE:'ORG_ALL_CACHE'};
	justep.Component.register(url, OrgDialogPC);
	
	justep.Component.addOperations(OrgDialogPC, {
		'open' : {
			label : "",
			argsDef: [{name:'option',displayName:justep.Message.getMessage(justep.Message.JUSTEP231098)}],
			method : function(args) {
				return this.owner.open(args.option);
			}
		}
	});
	
	return OrgDialogPC;
});